package com.example.tdl.service;

import com.example.tdl.config.Influxdbconfig;
import org.influxdb.InfluxDB;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.influxdb.dto.QueryResult.Result;
import org.influxdb.dto.QueryResult.Series;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service("influxdbService")
public class InfluxDBService {
    @Autowired
    Influxdbconfig influxdbconfig;

    @Autowired
    private InfluxDB influxDB;

    public final static String DEFAULT_DATABASE = "original";
    public final static String DEFAULT_POLICY = "witium_policy";
    private final static String DEFAULT_DURATION = "0d";
    private final static String DEFAULT_SHARD_DURATION = "90d";
    
    private String   mPolicyName = DEFAULT_POLICY;
    private String   mDBName = DEFAULT_DATABASE;
    public void  connectDatabase(String dbname) {
        if(!dbname.isEmpty() && dbname != null)
        {
            useDatabase(dbname);
            useRetentionPolicy(mPolicyName);
        }
    }

    /* Create the database if it is not exist. */
    private void createDatabase(String database) {
        if (database != null && !database.isEmpty()) {
            if (!influxDB.databaseExists(database)) {
                influxDB.createDatabase(database);
            }
        }
    }

    public void useDatabase(String database) {
        if (!influxDB.databaseExists(database)) {
            createDatabase(database);
        }
        influxdbconfig.setDatabase(database);
        mDBName = database;
    }

    public List<String> showRetentionPolicys() {
        String command = String.format("SHOW RETENTION POLICIES ON \"%s\"",
                mDBName);
        QueryResult results = this.query(command);
        List<String> policys = new ArrayList<String>();

        for (Result result : results.getResults()) {
            List<Series> series = result.getSeries();
            for (Series serie : series) {
                List<List<Object>> values = serie.getValues();
                for (List<Object> value : values) {
                    policys.add(0, (String)value.get(0));
                }
            }
        }
        return policys;
    }

    /* Judge whether policy is exist. */
    public boolean isRetentionPolicyExist(String policyname) {
        boolean isExist = false;
        List<String> policys;
        policys = showRetentionPolicys();
        for (String policy : policys) {
            if (policy.equals(policyname)) {
                isExist = true;
                break;
            }
        }
        return isExist;
    }

    public void useRetentionPolicy(String policy) {
        mPolicyName = policy;
        if (!isRetentionPolicyExist(policy)) {
            createRetentionPolicy(DEFAULT_DURATION, DEFAULT_SHARD_DURATION, 1, true);
        }
    }

    /**
     * 创建保存策略 <br/>
     * CREATE RETENTION POLICY "default" ON "database" DURATION 30d REPLICATION 1 DEFAULT
     *
     * @param duration 存放时间
     * @param replicationNum 备份数量
     */
    public void createRetentionPolicy(String duration, Integer replicationNum, Boolean isDefault) {

        if (mPolicyName != null && !mPolicyName.isEmpty()
                && mDBName != null && !mDBName.isEmpty()) {
            String command = String.format("CREATE RETENTION POLICY \"%s\" ON \"%s\" DURATION %s REPLICATION %s %s",
                    mPolicyName, mDBName, duration, replicationNum, isDefault ? "DEFAULT" : "");
            this.query(command);
        }
    }
    
    /**
     * 创建保存策略 <br/>
     * CREATE RETENTION POLICY "default" ON "database" DURATION 30d SHARD DURATION %s REPLICATION 1 DEFAULT
     *
     * @param duration 存放时间
     * @param shardGroupDuration 存放快速读取组时间
     * @param replicationNum 备份数量
     */
    public void createRetentionPolicy(String duration, String shardGroupDuration, Integer replicationNum,
    		Boolean isDefault) {

        if (mPolicyName != null && !mPolicyName.isEmpty()
                && mDBName != null && !mDBName.isEmpty()) {
            String command = String.format("CREATE RETENTION POLICY \"%s\" ON \"%s\" DURATION %s SHARD DURATION %s REPLICATION %s %s",
                    mPolicyName, mDBName, duration, shardGroupDuration, replicationNum, isDefault ? "DEFAULT" : "");
            this.query(command);
        }
    }

    /**
     * 插入数据
     *
     * @param measurement a Point in a fluent manner
     * @param tagsToAdd the Map of tags to add
     * @param fields the fields to add
     */
    public void insert(String measurement, Long timestamp, Map<String, String> tagsToAdd, Map<String, Object> fields) {

        if (measurement != null && !measurement.isEmpty()
                && tagsToAdd != null && !tagsToAdd.isEmpty()
                && fields != null && !fields.isEmpty()) {
            Point.Builder builder = Point.measurement(measurement)
                    .time(timestamp, TimeUnit.MILLISECONDS)
                    .tag(tagsToAdd)
                    .fields(fields);
            try {
                influxDB.write(mDBName, mPolicyName, builder.build());
            } catch (RuntimeException e) {
                // TODO: handle exception
                e.printStackTrace();
            } catch (Exception e) {
                // TODO: handle exception
                e.printStackTrace();
            }
        }
    }

    public void insertBatchWithMultiTags(String dbname, String measurement, Long[] timestamp,
    		List<Map<String, String>> tagsToAdd, List<Map<String, Object>> fields)
    {
        if (measurement != null && !measurement.isEmpty()
                && tagsToAdd != null && !tagsToAdd.isEmpty()
                && fields != null && !fields.isEmpty()) {

            BatchPoints batchPoints = BatchPoints.database(dbname)
                    .retentionPolicy(mPolicyName)
                    .consistency(InfluxDB.ConsistencyLevel.ALL)
                    .build();

            for (int i = 0; i < tagsToAdd.size(); i++)
            {
                Point pointdata = Point.measurement(measurement)
                        .time(timestamp[i], TimeUnit.MILLISECONDS)
                        .tag(tagsToAdd.get(i))
                        .fields(fields.get(i))
                        .build();
                batchPoints.point(pointdata);
            }
            try {
                influxDB.write(batchPoints);
            } catch (RuntimeException e) {
                // TODO: handle exception
                e.printStackTrace();
            } catch (Exception e) {
                // TODO: handle exception
                e.printStackTrace();
            }
        }
    }

    public void insertBatchWithSameTags(String dbname, String measurement, Long[] timestamp,
    		Map<String, String> tagsToAdd, List<Map<String, Object>> fields)
    {
        if (measurement != null && !measurement.isEmpty()
                && tagsToAdd != null && !tagsToAdd.isEmpty()
                && fields != null && !fields.isEmpty()) {

            BatchPoints batchPoints = BatchPoints.database(dbname)
                    .retentionPolicy(mPolicyName)
                    .consistency(InfluxDB.ConsistencyLevel.ALL)
                    .build();

            for (int i = 0; i < fields.size(); i++)
            {
                Point pointdata = Point.measurement(measurement)
                            .time(timestamp[i], TimeUnit.MILLISECONDS)
                            .tag(tagsToAdd)
                            .fields(fields.get(i))
                            .build();

                if(pointdata != null)
                {
                    batchPoints.point(pointdata);
                }
            }

            try {
                influxDB.write(batchPoints);
            } catch (RuntimeException e) {
                // TODO: handle exception
                e.printStackTrace();
            } catch (Exception e) {
                // TODO: handle exception
                e.printStackTrace();
            }
        }
    }

    /**
     * 查询数据
     *
     * @param command
     * @return QueryResult
     */
    public QueryResult query(String command) {
        return influxDB.query(new Query(command, mDBName));
    }


}
